<?php

App::uses('CakeEventListener', 'Event');

class EvBasketCheckForDiscountListener implements CakeEventListener {

	public function implementedEvents() {
		return array(
			'EvDiscount.Component.DiscountCodes.codeAdded' => 'addDiscount',
			'EvBasket.Component.Basket.itemAdd' => 'rebuildDiscount',
			'EvBasket.Component.Basket.itemUpdated' => 'rebuildDiscount',
			'EvBasket.Component.Basket.itemDeleted' => 'rebuildDiscount'
		);
	}

	/**
	 * queue up the confirmation emails
	 *
	 * @param 	CakeEvent
	 */
	public function addDiscount(CakeEvent $Event) {
		// Event subject will be the EvDiscount.DiscountCodesComponent
		return $this->rebuildDiscount($Event);
	}

	/**
	 * rebuild discount due to basket amends
	 *
	 * @param 	CakeEvent
	 */
	public function rebuildDiscount(CakeEvent $Event) {
		if (! CakePlugin::loaded('EvDiscount')) {
			return false;
		}

		App::uses('DiscountLib', 'EvDiscount.Lib');

		$DiscountCode = EvClassRegistry::init('EvDiscount.DiscountCode');

		$EventSubject = $Event->subject();

		if (! is_object($EventSubject->_controller->DiscountCookie)) {
			$EventSubject->_controller->DiscountCookie = $EventSubject->_controller->loadComponent('EvDiscount.DiscountCookie');
		}

		// try to read the cookies
		$codeCookie = $EventSubject->_controller->DiscountCookie->read('Code');

		if (! empty($codeCookie) && is_array($codeCookie)) {
			$validCodes = $DiscountCode->validateCodes($codeCookie);
		}

		if (! is_object($EventSubject->_controller->BasketManager)) {
			$EventSubject->_controller->BasketManager = $EventSubject->_controller->loadComponent('EvBasket.BasketManager');
		}

		$total = $EventSubject->_controller->BasketManager->findTotalRow(
			Configure::read('EvBasket.labels.subtotal')
		);
		
		$grandTotal = $EventSubject->_controller->BasketManager->findTotalRow(
			Configure::read('EvBasket.labels.grandtotal')
		);

		$taxTotal = $EventSubject->_controller->BasketManager->findTotalRow(
			Configure::read('EvBasket.labels.tax')
		);

		$discount = 0;

		if (! empty($validCodes) && is_array($validCodes)) {
			foreach ($validCodes as $codeData) {
				if (!empty($codeData['DiscountCode']['min_order']) && $total['amount'] < $codeData['DiscountCode']['min_order']) {
					continue;
				}

				$amount = DiscountLib::calculate($codeData, $total['amount']);

				if (! empty($amount) && Configure::read('SiteSetting.ev_basket.discounts_include_tax')) {
					$basketTaxRate = number_format($taxTotal['amount'] / ($grandTotal['amount'] - $taxTotal['amount']), 2);

					$discountTax = number_format($amount - ($amount / (1 + $basketTaxRate)), 2);
					$discountPreTax = $amount - $discountTax;
					$discount += $discountPreTax;
				}
			}
		}

		if (! empty($discount) && $discount > 0) {
			$EventSubject->_controller->BasketManager->manageTotalRow(
				Configure::read('EvBasket.labels.discount'),
				-$discount,
				15
			);
			
			if (Configure::read('SiteSetting.ev_basket.discounts_include_tax')) {
				$EventSubject->_controller->BasketManager->manageTotalRow(
					Configure::read('EvBasket.labels.tax'),
					($taxTotal['amount'] - $discountTax),
					30
				);
			}

			$Event->subject()->_controller->BasketManager->rebuildTotals();
		}

		return true;
	}
}
